import { IDisposable, Disposable, Event, Emitter } from '@newstudios/common';
import { MimeType, Source } from '../types';
export interface QualityLevel {
    readonly bitrate: number;
    readonly width: number;
    readonly height: number;
    readonly type?: 'video' | 'audio' | 'image';
    readonly fps?: number;
}
export declare type PlayList = readonly QualityLevel[];
export interface SourceWithMimeType extends Readonly<Source> {
    readonly mime: MimeType;
}
export interface ICorePlayer extends IDisposable {
    /** 当前播放器内核名称 */
    readonly name: string;
    /** 当 PlayList 发生改变时触发 */
    readonly onPlayListChange: Event<PlayList>;
    /** 当选中一个 QualityID 时触发 */
    readonly onQualityIdSelect: Event<string>;
    /** 当播放质量切换发生请求时触发 */
    readonly onQualitySwitching: Event<QualityLevel>;
    /** 当播放质量切换完成时触发 */
    readonly onQualityChange: Event<QualityLevel>;
    /** 当自动清晰度选项切换时触发 */
    readonly onAutoChange: Event<boolean>;
    /** 当播放器初始化完成，可以切换播放质量时触发 */
    readonly onReady: Event<void>;
    /** 实际的当前播放质量级别，未确定时为 undefined */
    readonly qualityLevel?: QualityLevel;
    /** 实际的准备切换的播放质量级别，未确定时为 undefined */
    readonly nextQualityLevel?: QualityLevel;
    /** 当前已知的所有质量级别组 */
    readonly playList: PlayList;
    /** 当前是否启用了 auto 切换质量 */
    readonly autoQuality: boolean;
    /** 实际的当前播放质量ID，未确定时为 auto */
    readonly qualityId: string;
    /** 当前选择的质量ID，未确定时为 auto */
    readonly selectedQualityId: string;
    /** 实际的下一个播放质量ID，自动时时为 auto */
    readonly nextQualityId: string;
    /** 告知当前播放内容是否支持自动质量切换 */
    readonly supportAutoQuality: boolean;
    /** 是否初始化完成 playList，并准备就绪 */
    readonly ready: boolean;
    /** 获取估算的带宽 */
    readonly bandwidthEstimate: number;
    /** 获取是否 cap to player */
    readonly capLevelToPlayerSize: boolean;
    /** 根据质量ID设定播放质量，ID 为 auto 表示自动切换 */
    setQualityById(id: string): void;
    /** 初始的播放比特率 */
    setInitialBitrate(bitrate: number): void;
    /** 设置 cap to player */
    setCapLevelToPlayerSize(capLevelToPlayer: boolean): void;
    log(...messages: any[]): void;
}
/**
 * 将 id 转变为 Quality Level
 * @param id 指定的 ID，形如 br2000000-1920x1080-video / br1200000-1280x720
 */
export declare function idToQualityLevel(id: string): QualityLevel | undefined;
/**
 * 将播放质量级别转为为播放质量 ID
 * @param level 播放质量级别
 */
export declare function qualityLevelToId(level: QualityLevel): string;
/** 播放质量 ID 是否为 auto 自动切换 */
export declare function isAutoQuality(id: string): id is 'auto';
/** 将任意 fps 字段转化为整数 fps，如果失败则返回 NaN */
export declare function computeFPS(fps: string | undefined | number | null): number;
/** 两个播放质量是否同级 */
export declare function isSameLevel(level1: QualityLevel | undefined | null, level2: QualityLevel | undefined | null): boolean;
export declare abstract class CorePlayer<Level = unknown> extends Disposable implements ICorePlayer {
    protected readonly video: HTMLVideoElement;
    protected readonly source: SourceWithMimeType;
    protected readonly _onPlayListChange: Emitter<PlayList>;
    readonly onPlayListChange: Event<PlayList>;
    protected readonly _onQualityIdSelect: Emitter<string>;
    readonly onQualityIdSelect: Event<string>;
    protected readonly _onQualityChange: Emitter<QualityLevel>;
    readonly onQualityChange: Event<QualityLevel>;
    protected readonly _onQualitySwitching: Emitter<QualityLevel>;
    readonly onQualitySwitching: Event<QualityLevel>;
    protected readonly _onAutoChange: Emitter<boolean>;
    readonly onAutoChange: Event<boolean>;
    protected readonly _onReady: Emitter<void>;
    readonly onReady: Event<void>;
    protected readonly onOncePlayListReady: Event<PlayList>;
    private _playList;
    private _qualityLevel?;
    private _nextQualityLevel?;
    private _autoQuality?;
    private _ready;
    private _selectedQualityId;
    private _onPlayListMutable;
    abstract get name(): string;
    abstract get supportAutoQuality(): boolean;
    abstract get bandwidthEstimate(): number;
    abstract setInitialBitrate(bitrate: number): void;
    abstract get capLevelToPlayerSize(): boolean;
    abstract setCapLevelToPlayerSize(capLevelToPlayer: boolean): void;
    /** debug flag */
    debug: boolean;
    protected abstract get levels(): Level[];
    protected abstract get currentLevel(): Level | undefined;
    protected abstract get nextLevel(): Level | undefined;
    protected abstract get autoQualityEnabled(): boolean;
    protected abstract levelToQuality(level: Level): QualityLevel;
    protected abstract findLevelIndexByQualityLevel(level: QualityLevel): number;
    protected abstract setAutoQualityState(auto: boolean): void;
    protected abstract setNextLevelIndex(index: number): void;
    /** 实现 video 和播放 src 对应的初始化关系，该 src 通常为一个 mp4 或 m3u8 或 mpd */
    protected abstract onInit(video: HTMLVideoElement, source: SourceWithMimeType): void;
    constructor(video: HTMLVideoElement, source: SourceWithMimeType);
    log(...args: any): void;
    get playList(): PlayList;
    /** 更新 PlayList 播放级别组，每当获取到新的 PlayList 时请调用此接口 */
    protected updatePlayList(): void;
    get nextQualityLevel(): QualityLevel | undefined;
    /** 更新下一个播放质量级别，每当发生播放质量切换开始时请调用此接口 */
    protected updateNextQualityLevel(): void;
    get qualityLevel(): QualityLevel | undefined;
    /** 更新当前播放质量级别，每当发生播放质量切换结束时请调用此接口 */
    protected updateQualityLevel(): void;
    get autoQuality(): boolean;
    /** 每当切换自动清晰度状态时，请调用此接口 */
    protected updateAutoQuality(): void;
    get ready(): boolean;
    /** 更新当前播放器状态，每当初始化完成后情调用此接口 */
    protected setReady(): void;
    get selectedQualityId(): string;
    get qualityId(): string;
    get nextQualityId(): string;
    /**
     * @FIXME should find a id similar to playlist in coreplayer
     * @param id any quality id
     */
    setQualityById(id: string): void;
    protected levelIndexToQualityId(level: number): string;
    private findLevelIndexById;
    /** 翻译当前 PlayList */
    private translatePlayList;
    /** 翻译当前 QualityLevel */
    private translateCurrentQuality;
    /** 翻译下一个 QualityLevel, 当前 autoQuality 打开时返回 undefined */
    private translateNextQualityLevel;
    private setPlayList;
    private setNextQualityLevel;
    private setQualityLevel;
}
